"""
Configs migration

Generation datetime: 2021-07-19 10:39:22.539095
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper as mh, LimitationToModify

REVISION = "1d02fbdf"
DOWN_REVISION = "f4c31695"


limitationsToModify: List[LimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="PLATFORM_LIMITS",
        validationSchema={
            "type": "object",
            "properties": {
                "match": {
                    "type": "object",
                    "properties": {
                        "array_filter_limit": {"type": "integer", "minimum": 1},
                        "reference_limit": {"type": "integer", "minimum": 1},
                        "candidate_limit": {"type": "integer", "minimum": 1},
                        "result_candidate_limit": {"type": "integer", "minimum": 1},
                    },
                    "additionalProperties": False,
                },
                "cross_match": {
                    "type": "object",
                    "properties": {
                        "short_array_filter_limit": {"type": "integer", "minimum": 1},
                        "array_filter_limit": {"type": "integer", "minimum": 1},
                        "result_candidate_limit": {"type": "integer", "minimum": 1},
                        "general_limit": {"type": "integer", "minimum": 1},
                    },
                    "additionalProperties": False,
                },
            },
            "additionalProperties": False,
        },
        defaultValue={
            "match": {
                "array_filter_limit": 1000,
                "reference_limit": 30,
                "candidate_limit": 30,
                "result_candidate_limit": 100,
            },
            "cross_match": {
                "short_array_filter_limit": 1000,
                "array_filter_limit": 20000,
                "result_candidate_limit": 20000,
                "general_limit": 100000,
            },
        },
        description="platform limits",
        services=["luna-matcher-proxy", "luna-python-matcher", "luna-tasks"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="PLATFORM_LIMITS")
