"""
Configs migration

Generation datetime: 2023-01-10 13:54:13.159697
"""

from typing import List

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "61f4d214"
DOWN_REVISION = "6bdadd3f"


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="INFLUX_MONITORING",
        oldValidationSchema={
            "properties": {
                "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                "use_ssl": {"type": "integer", "enum": [0, 1]},
                "flushing_period": {"type": "number"},
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "organization": {"type": "string"},
                "token": {"type": "string"},
                "bucket": {"type": "string"},
            },
            "default": {
                "send_data_for_monitoring": 1,
                "use_ssl": 0,
                "flushing_period": 1,
                "host": "127.0.0.1",
                "port": 8086,
                "organization": "luna",
                "token": "secret-token",
                "bucket": "luna_monitoring",
            },
            "required": [
                "send_data_for_monitoring",
                "host",
                "port",
                "organization",
                "token",
                "bucket",
            ],
            "type": "object",
            "additionalProperties": False,
            "description": "Monitoring settings.",
        },
        newValidationSchema={
            "properties": {
                "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                "use_ssl": {"type": "integer", "enum": [0, 1]},
                "flushing_period": {"type": "number"},
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "organization": {"type": "string"},
                "token": {"type": "string"},
                "bucket": {"type": "string"},
            },
            "default": {
                "send_data_for_monitoring": 1,
                "use_ssl": 0,
                "flushing_period": 1,
                "host": "127.0.0.1",
                "port": 8086,
                "organization": "luna",
                "token": "secret-token",
                "bucket": "luna_monitoring",
            },
            "required": [
                "send_data_for_monitoring",
                "host",
                "port",
                "organization",
                "token",
                "bucket",
            ],
            "type": "object",
            "additionalProperties": False,
            "description": "Monitoring settings.",
        },
        oldDefaultValue={
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        newDefaultValue={
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        oldDescription="monitoring parameters",
        newDescription="monitoring parameters",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-index-manager",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-sender",
            "luna-tasks",
        ],
        newServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-sender",
            "luna-tasks",
        ],
    ),
    LimitationToModify(
        limitationName="STORAGE_TIME",
        oldValidationSchema={"type": "string", "enum": ["LOCAL", "UTC"]},
        newValidationSchema={"type": "string", "enum": ["LOCAL", "UTC"]},
        oldDefaultValue="LOCAL",
        newDefaultValue="LOCAL",
        oldDescription="Time in which records are stored in the database",
        newDescription="Time in which records are stored in the database",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-accounts",
            "luna-backport3",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-index-manager",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
        newServices=[
            "luna-accounts",
            "luna-backport3",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
    ),
    LimitationToModify(
        limitationName="ADDITIONAL_SERVICES_USAGE",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "indexed_matcher": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
            },
            "required": [
                "luna_sender",
                "luna_events",
                "luna_tasks",
                "indexed_matcher",
                "luna_matcher_proxy",
            ],
            "description": "whether to use nested optional services",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
            },
            "required": [
                "luna_sender",
                "luna_events",
                "luna_tasks",
                "luna_matcher_proxy",
            ],
            "description": "whether to use nested optional services",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_tasks": True,
            "indexed_matcher": False,
            "luna_matcher_proxy": False,
        },
        newDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_tasks": True,
            "luna_matcher_proxy": False,
        },
        oldDescription="whether to use nested optional services",
        newDescription="whether to use nested optional services",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=lambda x: {service: x[service] for service in x if service != "indexed_matcher"},
        settingDowngradeFunction=lambda x: (x.update({"indexed_matcher": False}), x)[1],
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-tasks",
        ],
    ),
    LimitationToModify(
        limitationName="LUNA_FACES_ADDRESS",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "origin": {
                    "type": "string",
                    "pattern": "^http(s)?:\\/\\/[^\\/:]+(:\\d+)?$",
                    "example": "http://127.0.0.1:5051",
                },
                "api_version": {"type": "integer"},
            },
            "required": ["origin", "api_version"],
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "origin": {
                    "type": "string",
                    "pattern": "^http(s)?:\\/\\/[^\\/:]+(:\\d+)?$",
                    "example": "http://127.0.0.1:5051",
                },
                "api_version": {"type": "integer"},
            },
            "required": ["origin", "api_version"],
            "additionalProperties": False,
        },
        oldDefaultValue={"origin": "http://127.0.0.1:5030", "api_version": 3},
        newDefaultValue={"origin": "http://127.0.0.1:5030", "api_version": 3},
        oldDescription="LUNA Faces address",
        newDescription="LUNA Faces address",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-backport4",
            "luna-handlers",
            "luna-index-manager",
            "luna-indexed-matcher",
            "luna-indexer",
            "luna-tasks",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-backport4",
            "luna-handlers",
            "luna-tasks",
        ],
    ),
    LimitationToModify(
        limitationName="LUNA_FACES_TIMEOUTS",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "request": {"type": "number", "exclusiveMinimum": 0},
                "connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_read": {"type": "number", "exclusiveMinimum": 0},
            },
            "description": "timeouts in seconds",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "request": {"type": "number", "exclusiveMinimum": 0},
                "connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_read": {"type": "number", "exclusiveMinimum": 0},
            },
            "description": "timeouts in seconds",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "request": 60,
            "connect": 20,
            "sock_connect": 10,
            "sock_read": 60,
        },
        newDefaultValue={
            "request": 60,
            "connect": 20,
            "sock_connect": 10,
            "sock_read": 60,
        },
        oldDescription="timeouts for http-request to luna-faces service",
        newDescription="timeouts for http-request to luna-faces service",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-api",
            "luna-backport4",
            "luna-handlers",
            "luna-index-manager",
            "luna-tasks",
        ],
        newServices=["luna-api", "luna-backport4", "luna-handlers", "luna-tasks"],
    ),
]

externalLimitationsToModify: List[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # groups to remove
    mh.removeGroup(groupName="luna-queue")

    # limitations to remove
    mh.removeLimitation(limitationName="CRAWLER_PERIOD")
    mh.removeLimitation(limitationName="INDEXED_FACES_LISTS")
    mh.removeLimitation(limitationName="INDEXED_LISTS_REFRESH_RATE")
    mh.removeLimitation(limitationName="INDEXED_MATCHER_FACES_CLIENT_PROPERTIES")
    mh.removeLimitation(limitationName="INDEXED_MATCHER_FSDK")
    mh.removeLimitation(limitationName="INDEXED_MATCHER_QUEUE")
    mh.removeLimitation(limitationName="INDEXED_MATCHER_WORKER_AND_MESSAGE")
    mh.removeLimitation(limitationName="INDEXED_MATCHING")
    mh.removeLimitation(limitationName="INDEXER_FACES_CLIENT_PROPERTIES")
    mh.removeLimitation(limitationName="INDEXER_FSDK")
    mh.removeLimitation(limitationName="INDEXER_QUEUE")
    mh.removeLimitation(limitationName="INDEXER_WORKER_AND_MESSAGE")
    mh.removeLimitation(limitationName="INDEXING")
    mh.removeLimitation(limitationName="INDEX_PATH")
    mh.removeLimitation(limitationName="INDEX_RELOADED_FOLDER")
    mh.removeLimitation(limitationName="LUNA_INDEX_MANAGER_ACTIVE_PLUGINS")
    mh.removeLimitation(limitationName="LUNA_INDEX_MANAGER_ADDRESS")
    mh.removeLimitation(limitationName="LUNA_INDEX_MANAGER_DB")
    mh.removeLimitation(limitationName="LUNA_INDEX_MANAGER_LOGGER")
    mh.removeLimitation(limitationName="LUNA_INDEX_MANAGER_TIMEOUTS")
    mh.removeLimitation(limitationName="LUNA_MATCHERS_DAEMONS_ORIGINS")
    mh.removeLimitation(limitationName="LUNA_MATCHER_CLUSTERS")
    mh.removeLimitation(limitationName="LUNA_MATCHER_DAEMON_TIMEOUTS")
    mh.removeLimitation(limitationName="MATCHER_DAEMON")
    mh.removeLimitation(limitationName="MIN_FACES_IN_LIST_FOR_INDEXING")
    mh.removeLimitation(limitationName="QUEUE_SETTINGS")

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # groups to add
    mh.addGroup(groupName="luna-queue", groupDescription="")

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to add
    mh.addLimitation(
        limitationName="CRAWLER_PERIOD",
        validationSchema={"type": "number"},
        defaultValue=1,
        description="luna-faces polling frequency, in minutes",
        services=["luna-index-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXED_FACES_LISTS",
        validationSchema={"type": "array", "items": {"type": "string"}, "minItems": 0},
        defaultValue=[],
        description="lists for indexation, for value ['all'] will be indexed all lists with attributes count >= MIN_FACES_IN_LIST_FOR_INDEXING",
        services=["luna-index-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXED_LISTS_REFRESH_RATE",
        validationSchema={"type": "integer"},
        defaultValue=1,
        description="indexed lists refresh rate, in seconds, works if USE_INDEX_MANAGER",
        services=["luna-matcher-proxy"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXED_MATCHER_FACES_CLIENT_PROPERTIES",
        validationSchema={
            "type": "object",
            "properties": {
                "httpClient": {
                    "type": "object",
                    "properties": {
                        "requestTimeoutMs": {"type": "integer"},
                        "connectTimeoutMs": {"type": "integer"},
                    },
                    "description": "Timeouts settings.",
                }
            },
        },
        defaultValue={"httpClient": {"requestTimeoutMs": 5000, "connectTimeoutMs": 2000}},
        description="Indexed matcher faces client properties.",
        services=["luna-indexed-matcher"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXED_MATCHER_FSDK",
        validationSchema={
            "type": "object",
            "properties": {
                "fsdk": {
                    "type": "object",
                    "properties": {
                        "configPath": {"type": "string"},
                        "runtimeConfigPath": {"type": "string"},
                        "dataPath": {"type": "string"},
                    },
                    "description": "Paths to fsdk config and data.",
                }
            },
        },
        defaultValue={
            "fsdk": {
                "configPath": "/srv/luna/conf/faceengine.conf",
                "runtimeConfigPath": "/srv/luna/conf/runtime.conf",
                "dataPath": "/srv/luna/data",
            }
        },
        description="Indexed matcher fsdk settings.",
        services=["luna-indexed-matcher"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXED_MATCHER_QUEUE",
        validationSchema={
            "type": "object",
            "properties": {
                "routing_key": {"type": "string"},
                "rabbit_exchange": {"type": "string"},
            },
            "required": ["routing_key"],
            "additionalProperties": False,
        },
        defaultValue={
            "routing_key": "indexed_matcher",
            "rabbit_exchange": "luna.match",
        },
        description="Indexed Matcher queue settings.",
        services=["luna-admin", "luna-indexed-matcher"],
        groupNames=["luna-platform", "luna-queue"],
    )
    mh.addLimitation(
        limitationName="INDEXED_MATCHER_WORKER_AND_MESSAGE",
        validationSchema={
            "type": "object",
            "properties": {
                "msg": {"type": "object", "properties": {"path": {"type": "string"}}},
                "worker": {
                    "type": "object",
                    "properties": {
                        "connection": {
                            "type": "object",
                            "properties": {
                                "enableDLQ": {"type": "boolean"},
                                "qos": {"type": "integer"},
                                "disableAck": {"type": "boolean"},
                                "produceOnly": {"type": "boolean"},
                            },
                        }
                    },
                },
            },
            "description": "Worker and messaging settings.",
        },
        defaultValue={
            "msg": {"path": ""},
            "worker": {
                "connection": {
                    "enableDLQ": False,
                    "qos": 1,
                    "disableAck": False,
                    "produceOnly": False,
                }
            },
        },
        description="Indexed matcher worker and messaging settings.",
        services=["luna-indexed-matcher"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXED_MATCHING",
        validationSchema={
            "type": "object",
            "properties": {
                "cache": {
                    "type": "object",
                    "properties": {
                        "capacity": {"type": "integer"},
                        "policy": {"type": "string", "enum": ["classic", "warmup"]},
                        "skipList": {"type": "array", "items": {"type": "string"}},
                    },
                },
                "listsLoadExternalIds": {"type": "boolean"},
                "matcherStrategy": {
                    "type": "string",
                    "enum": ["lists", "indexes", "events"],
                },
                "staticBatchCapacity": {"type": "integer"},
                "staticBatchVersionCheckPeriod": {"type": "integer"},
                "matchingThreadCheckPeriodCoeff": {"type": "number"},
                "warmupPreloadPerPageRetries": {"type": "integer"},
                "lists_data_source": {"type": "string", "enum": ["rawfaces_api"]},
                "ef_search": {"type": "integer"},
                "goldengate_support": {"type": "boolean"},
                "onReload": {
                    "type": "object",
                    "properties": {
                        "logging": {
                            "type": "object",
                            "properties": {
                                "log-to-stderr": {"type": "boolean"},
                                "log-also-to-stderr": {"type": "boolean"},
                                "log-verbosity": {"type": "integer"},
                                "log-min-level": {"type": "integer"},
                                "log-buf-sec": {"type": "integer"},
                                "log-buf-level": {"type": "integer"},
                            },
                        }
                    },
                },
                "eventsInsertTimeDeltaHandicapSeconds": {
                    "type": "integer",
                    "minimum": 0,
                },
            },
            "description": "Matcher settings.",
        },
        defaultValue={
            "matcherStrategy": "indexes",
            "staticBatchCapacity": 5000,
            "staticBatchVersionCheckPeriod": 500,
            "lists_data_source": "rawfaces_api",
            "ef_search": 1600,
            "goldengate_support": False,
            "onReload": {
                "logging": {
                    "log-to-stderr": False,
                    "log-also-to-stderr": True,
                    "log-verbosity": 2,
                    "log-min-level": 0,
                    "log-buf-sec": 30,
                    "log-buf-level": 0,
                }
            },
        },
        description="Matcher settings.",
        services=["luna-indexed-matcher"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXER_FACES_CLIENT_PROPERTIES",
        validationSchema={
            "type": "object",
            "properties": {
                "httpClient": {
                    "type": "object",
                    "properties": {
                        "requestTimeoutMs": {"type": "integer"},
                        "connectTimeoutMs": {"type": "integer"},
                    },
                    "description": "Timeouts settings.",
                }
            },
        },
        defaultValue={"httpClient": {"requestTimeoutMs": 5000, "connectTimeoutMs": 2000}},
        description="Indexer faces client properties.",
        services=["luna-indexer"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXER_FSDK",
        validationSchema={
            "type": "object",
            "properties": {
                "fsdk": {
                    "type": "object",
                    "properties": {
                        "configPath": {"type": "string"},
                        "runtimeConfigPath": {"type": "string"},
                        "dataPath": {"type": "string"},
                    },
                    "description": "Paths to fsdk config and data.",
                }
            },
        },
        defaultValue={
            "fsdk": {
                "configPath": "/srv/luna/conf/faceengine.conf",
                "runtimeConfigPath": "/srv/luna/conf/runtime.conf",
                "dataPath": "/srv/luna/data",
            }
        },
        description="Indexer fsdk settings.",
        services=["luna-indexer"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXER_QUEUE",
        validationSchema={
            "type": "object",
            "properties": {
                "routing_key": {"type": "string"},
                "rabbit_exchange": {"type": "string"},
            },
            "required": ["routing_key"],
            "additionalProperties": False,
        },
        defaultValue={"routing_key": "indexer", "rabbit_exchange": "luna.index"},
        description="Indexer queue settings.",
        services=["luna-admin", "luna-index-manager", "luna-indexer"],
        groupNames=["luna-platform", "luna-queue"],
    )
    mh.addLimitation(
        limitationName="INDEXER_WORKER_AND_MESSAGE",
        validationSchema={
            "type": "object",
            "properties": {
                "msg": {"type": "object", "properties": {"path": {"type": "string"}}},
                "worker": {
                    "type": "object",
                    "properties": {
                        "connection": {
                            "type": "object",
                            "properties": {
                                "enableDLQ": {"type": "boolean"},
                                "qos": {"type": "integer"},
                                "disableAck": {"type": "boolean"},
                                "produceOnly": {"type": "boolean"},
                            },
                        }
                    },
                },
            },
            "description": "Worker and messaging settings.",
        },
        defaultValue={
            "msg": {"path": ""},
            "worker": {
                "connection": {
                    "enableDLQ": False,
                    "qos": 1,
                    "disableAck": False,
                    "produceOnly": False,
                }
            },
        },
        description="Indexer worker and messaging settings",
        services=["luna-indexer"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEXING",
        validationSchema={
            "type": "object",
            "properties": {
                "index": {
                    "type": "object",
                    "properties": {
                        "indexPeriodHrs": {"type": "integer"},
                        "concurrentFetch": {"type": "boolean"},
                        "store_index_max_days": {"type": "integer"},
                        "ef_construction": {"type": "integer"},
                    },
                    "description": "Indexer settings.",
                }
            },
            "required": ["index"],
        },
        defaultValue={
            "index": {
                "indexPeriodHrs": -1,
                "concurrentFetch": True,
                "store_index_max_days": -1,
                "ef_construction": 1600,
            }
        },
        description="Indexer settings.",
        services=["luna-indexer"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEX_PATH",
        validationSchema={
            "type": "object",
            "properties": {"indexPath": {"type": "string"}},
            "required": ["indexPath"],
            "description": "The path to which the constructed indices will be stored.",
        },
        defaultValue={"indexPath": "/var/lib/luna/index"},
        description="The path to which the constructed indices will be stored.",
        services=["luna-indexer", "luna-matcher-daemon"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="INDEX_RELOADED_FOLDER",
        validationSchema={
            "type": "object",
            "properties": {"indices_folder": {"type": "string"}},
            "required": ["indices_folder"],
            "description": "The path to the directory where the indices are located.",
        },
        defaultValue={"indices_folder": "/dev/shm/index_reload"},
        description="The path to the directory where the indices are located.",
        services=["luna-indexed-matcher", "luna-matcher-daemon"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_INDEX_MANAGER_ACTIVE_PLUGINS",
        validationSchema={"type": "array", "items": {"type": "string"}},
        defaultValue=[],
        description="list active plugins of luna-index-manager",
        services=["luna-index-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_INDEX_MANAGER_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "origin": {
                    "type": "string",
                    "pattern": "^http(s)?:\\/\\/[^\\/:]+(:\\d+)?$",
                    "example": "http://127.0.0.1:5051",
                },
                "api_version": {"type": "integer"},
            },
            "required": ["origin", "api_version"],
            "additionalProperties": False,
        },
        defaultValue={"origin": "http://127.0.0.1:5150", "api_version": 1},
        description="LUNA Index Manager address",
        services=["luna-admin", "luna-matcher-proxy"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_INDEX_MANAGER_DB",
        validationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": [
                "db_type",
                "db_user",
                "db_password",
                "db_name",
                "db_host",
                "db_port",
                "db_settings",
            ],
            "type": "object",
            "additionalProperties": False,
        },
        defaultValue={
            "db_type": "postgres",
            "db_user": "luna",
            "db_password": "luna",
            "db_name": "luna_index_manager",
            "db_host": "127.0.0.1",
            "db_port": 5432,
            "db_settings": {"connection_pool_size": 5},
        },
        description="Main database for luna-index-manager service",
        services=["luna-index-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_INDEX_MANAGER_LOGGER",
        validationSchema={
            "type": "object",
            "properties": {
                "log_level": {
                    "type": "string",
                    "enum": ["ERROR", "WARNING", "INFO", "DEBUG"],
                },
                "folder_with_logs": {"type": "string"},
                "max_log_file_size": {"type": "integer"},
                "log_time": {"type": "string", "enum": ["LOCAL", "UTC"]},
                "log_to_stdout": {"type": "boolean"},
                "log_to_file": {"type": "boolean"},
                "multiline_stack_trace": {"type": "boolean"},
            },
            "additionalProperties": False,
        },
        defaultValue={
            "log_level": "INFO",
            "folder_with_logs": "./",
            "max_log_file_size": 1024,
            "log_time": "LOCAL",
            "log_to_stdout": True,
            "log_to_file": True,
            "multiline_stack_trace": True,
        },
        description="Luna Index Manager logger settings",
        services=["luna-index-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_INDEX_MANAGER_TIMEOUTS",
        validationSchema={
            "type": "object",
            "properties": {
                "request": {"type": "number", "exclusiveMinimum": 0},
                "connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_read": {"type": "number", "exclusiveMinimum": 0},
            },
            "description": "timeouts in seconds",
            "additionalProperties": False,
        },
        defaultValue={
            "request": 60,
            "connect": 20,
            "sock_connect": 10,
            "sock_read": 60,
        },
        description="timeouts for http-request to luna-index-manager service",
        services=["luna-matcher-proxy"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_MATCHERS_DAEMONS_ORIGINS",
        validationSchema={
            "type": "array",
            "items": {"type": "string"},
            "minItems": 1,
            "description": "Matcher daemon endpoints.",
        },
        defaultValue=["http://127.0.0.1:6000/1"],
        description="list of luna-matcher-daemon endpoints",
        services=["luna-index-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_MATCHER_CLUSTERS",
        validationSchema={
            "type": "array",
            "description": "luna matcher clusters",
            "items": {
                "type": "object",
                "description": "core matcher cluster",
                "properties": {
                    "routing_key": {"type": "string"},
                    "rabbit_exchange": {"type": "string"},
                    "cluster_type": {
                        "type": "string",
                        "enum": ["core-indexed-matcher"],
                    },
                },
                "required": ["routing_key", "cluster_type"],
                "additionalProperties": False,
                "default": {
                    "cluster_type": "python-matcher",
                    "routing_key": "matcher",
                    "rabbit_exchange": "luna.match",
                },
            },
            "minItems": 0,
        },
        defaultValue=[
            {
                "cluster_type": "core-indexed-matcher",
                "routing_key": "matcher",
                "rabbit_exchange": "luna.match",
            }
        ],
        description="LUNA Matcher cluster list",
        services=["luna-matcher-proxy"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_MATCHER_DAEMON_TIMEOUTS",
        validationSchema={
            "type": "object",
            "properties": {
                "request": {"type": "number", "exclusiveMinimum": 0},
                "connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_read": {"type": "number", "exclusiveMinimum": 0},
            },
            "description": "timeouts in seconds",
            "additionalProperties": False,
        },
        defaultValue={
            "request": 60,
            "connect": 20,
            "sock_connect": 10,
            "sock_read": 60,
        },
        description="timeouts for http-request to luna-matcher-daemon service",
        services=["luna-index-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="MATCHER_DAEMON",
        validationSchema={
            "type": "object",
            "properties": {
                "indexer_host": {"type": "string"},
                "index_download_dir": {"type": "string"},
                "matcher_service": {"type": "string"},
                "ssh_key": {"type": "string"},
                "store_index_days": {"type": "integer"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
            },
            "required": [
                "indexer_host",
                "index_download_dir",
                "matcher_service",
                "store_index_days",
                "port",
            ],
            "description": "Matcher daemon settings.",
        },
        defaultValue={
            "indexer_host": "127.0.0.1",
            "index_download_dir": "/var/lib/matcher-daemon/index_download",
            "matcher_service": "luna-indexed-matcher",
            "ssh_key": "/etc/matcher-daemon/id_rsa",
            "store_index_days": 2,
            "port": 6001,
        },
        description="Matcher daemon settings.",
        services=["luna-matcher-daemon"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="MIN_FACES_IN_LIST_FOR_INDEXING",
        validationSchema={"type": "integer"},
        defaultValue=50000,
        description="min face count in list for indexing",
        services=["luna-index-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="QUEUE_SETTINGS",
        validationSchema={
            "type": "object",
            "description": "Core queue connection settings.",
            "oneOf": [
                {
                    "description": "Core Rabbit MQ settings.",
                    "type": "object",
                    "properties": {
                        "queue_type": {"type": "string", "enum": ["rabbit"]},
                        "max_retries": {"type": "integer"},
                        "delay_step": {"type": "integer"},
                        "api_version": {"type": "integer"},
                        "request_timeout": {"type": "number", "exclusiveMinimum": 0},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "user": {"type": "string"},
                        "password": {"type": "string"},
                    },
                    "required": [
                        "queue_type",
                        "api_version",
                        "host",
                        "port",
                        "user",
                        "password",
                    ],
                    "additionalProperties": False,
                    "default": {
                        "queue_type": "rabbit",
                        "max_retries": 5,
                        "delay_step": 10,
                        "request_timeout": 30,
                        "api_version": 14,
                        "host": "127.0.0.1",
                        "port": 5672,
                        "user": "luna",
                        "password": "luna",
                    },
                },
                {
                    "description": "Core IBM MQ settings.",
                    "type": "object",
                    "properties": {
                        "queue_type": {"type": "string", "enum": ["ibmmq"]},
                        "max_retries": {"type": "integer"},
                        "delay_step": {"type": "integer"},
                        "api_version": {"type": "integer"},
                        "request_timeout": {"type": "number", "exclusiveMinimum": 0},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "user": {"type": "string"},
                        "password": {"type": "string"},
                        "ibm_manager": {"type": "string"},
                        "ibm_channel": {"type": "string"},
                        "ibm_template": {"type": "string"},
                        "ibm_heart_beat_interval": {"type": "integer"},
                    },
                    "required": [
                        "queue_type",
                        "api_version",
                        "host",
                        "port",
                        "user",
                        "password",
                        "ibm_manager",
                        "ibm_channel",
                        "ibm_heart_beat_interval",
                        "ibm_template",
                    ],
                    "additionalProperties": False,
                    "default": {
                        "max_retries": 5,
                        "api_version": 14,
                        "queue_type": "ibmmq",
                        "host": "127.0.0.1",
                        "port": 1414,
                        "user": "luna",
                        "password": "luna",
                        "ibm_manager": "SATURN.QUEUE.MANAGER",
                        "ibm_channel": "PASSWORD.SVRCONN",
                        "ibm_template": "SYSTEM.DEFAULT.MODEL.QUEUE",
                        "ibm_heart_beat_interval": 1,
                    },
                },
                {
                    "description": "Core SQS settings.",
                    "type": "object",
                    "properties": {
                        "queue_type": {"type": "string", "enum": ["sqs"]},
                        "max_retries": {"type": "integer"},
                        "delay_step": {"type": "integer"},
                        "api_version": {"type": "integer"},
                        "request_timeout": {"type": "number", "exclusiveMinimum": 0},
                        "aws_public_access_key": {"type": "string"},
                        "aws_secret_access_key": {"type": "string"},
                        "aws_scheme": {
                            "type": "string",
                            "enum": ["http", "https", "MQTT"],
                        },
                        "aws_authorization_signature": {
                            "type": "string",
                            "enum": ["s3v2", "s3v4"],
                        },
                        "aws_region": {"type": "string"},
                        "aws_connect_timeout": {"type": "integer"},
                    },
                    "required": [
                        "queue_type",
                        "api_version",
                        "aws_public_access_key",
                        "aws_secret_access_key",
                        "aws_authorization_signature",
                    ],
                    "additionalProperties": False,
                    "default": {
                        "queue_type": "sqs",
                        "api_version": 1,
                        "aws_public_access_key": "",
                        "aws_secret_access_key": "",
                        "aws_scheme": "https",
                        "aws_authorization_signature": "s3v4",
                        "aws_region": "",
                        "aws_connect_timeout": 20,
                    },
                },
            ],
        },
        defaultValue={
            "queue_type": "rabbit",
            "max_retries": 5,
            "delay_step": 10,
            "request_timeout": 30,
            "api_version": 14,
            "host": "127.0.0.1",
            "port": 5672,
            "user": "luna",
            "password": "luna",
        },
        description="Determines queue connection settings.",
        services=[
            "luna-admin",
            "luna-index-manager",
            "luna-indexed-matcher",
            "luna-indexer",
            "luna-matcher-proxy",
        ],
        groupNames=["luna-platform", "luna-queue"],
    )
