import os
import sys
from pathlib import Path


def get_service_name() -> str:
    """
    Get service name.
    """
    service_name = os.getenv("SERVICE_NAME")
    if not service_name:
        service_name = str(Path(__file__).parent.parent.parent.parent.stem)
    service_name = service_name.replace("_", "-")
    return service_name


def get_service_path() -> Path:
    """
    Get service app folder.
    """
    service_name = get_service_name()
    path = service_name.replace("-", "_")
    return Path(path)


def solve_imports(level: int, file_name: str, path_remove_parent: bool = True) -> str:
    """
    Solution for importing external modules.
    Specifies the root of the package from which to resolve paths.
    """
    module = Path(file_name).resolve()
    parent = module.parent
    root = module.parents[level]
    sys.path.append(str(root))
    # it is necessary if there are import conflicts
    if path_remove_parent:
        try:
            sys.path.remove(str(parent))
        except ValueError:
            pass
    package = ".".join(parent.parts[len(root.parts) :])
    return package
