"""
Module contains tests for current dumps
"""

import copy
import decimal
import json
import unittest
from pathlib import Path

from configs_core.migration.consts import ACTUAL_DUMP_FILENAME
from configs_core.migration.generator import generateDump

from configs.settings.all_settings import LUNA_CONFIGURATOR, POSSIBLE_SERVICE_NAMES, SETTINGS


class TestCurrentDump(unittest.TestCase):
    """Compare current dump with actual settings from all_settings.py"""

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()

        serviceNames = copy.copy(POSSIBLE_SERVICE_NAMES)
        if LUNA_CONFIGURATOR in serviceNames:
            # remove configurator
            serviceNames.remove(LUNA_CONFIGURATOR)
        cls.dumpFromSettings = generateDump(set(serviceNames), SETTINGS)
        with open(Path(Path(__file__).parent.parent / "configs" / "migration" / ACTUAL_DUMP_FILENAME), "r") as dumpFile:
            cls.dumpFromJson = json.loads(dumpFile.read(), parse_float=decimal.Decimal())
        del cls.dumpFromJson["version"]

    def test_current_dump_equal_limitations(self):  # pylint: disable-msg=C0103
        """Compare limitations"""
        limitationMapFromJson = {row["limitation_name"]: row for row in self.dumpFromJson["limitations"]}
        limitationMapFromSettings = {row["limitation_name"]: row for row in self.dumpFromSettings["limitations"]}
        self.assertListEqual(
            sorted(list(limitationMapFromJson.keys())),
            sorted(list(limitationMapFromSettings.keys())),
            msg="Limitations names not equal",
        )
        for limitationsName in limitationMapFromJson:
            self.assertDictEqual(
                limitationMapFromJson[limitationsName], limitationMapFromSettings[limitationsName], limitationsName
            )

    def test_current_dump_equal_settings(self):  # pylint: disable-msg=C0103
        """Compare settings"""
        settingMapFromJson = {row["name"]: row for row in self.dumpFromJson["settings"]}
        settingMapFromSettings = {row["name"]: row for row in self.dumpFromSettings["settings"]}
        self.assertListEqual(
            sorted(list(settingMapFromJson.keys())),
            sorted(list(settingMapFromSettings.keys())),
            msg="Settings names not equal",
        )
        for settingName in settingMapFromJson:
            self.assertDictEqual(settingMapFromJson[settingName], settingMapFromSettings[settingName])

    def test_current_dump_equal_groups(self):  # pylint: disable-msg=C0103
        """Compare groups"""
        groupMapFromJson = {row["group_name"]: row for row in self.dumpFromJson["groups"]}
        groupMapFromSettings = {row["group_name"]: row for row in self.dumpFromSettings["groups"]}
        self.assertListEqual(
            sorted(list(groupMapFromJson.keys())),
            sorted(list(groupMapFromSettings.keys())),
            msg="Group names not equal",
        )
        for groupName in groupMapFromJson:
            self.assertDictEqual(groupMapFromJson[groupName], groupMapFromSettings[groupName])
