from argparse import ArgumentParser, ArgumentTypeError
from typing import Literal


def positiveInt(value):
    """Positive integer getter"""
    ivalue = int(value)
    if ivalue <= 0:
        raise ArgumentTypeError("%s is an invalid positive int value" % value)
    return ivalue


def getParsedCMDArgs(lambdaType: Literal["handlers", "tasks", "standalone"] = "handlers"):
    """Get parsed cmd args"""
    description = "Lambda application" if lambdaType == "handlers" else "Lambda tasks application"
    name = "lambda-unique-name" if lambdaType == "handlers" else "lambda-task-unique-name"
    nameHelp = "unique lambda name" if lambdaType == "handlers" else "unique lambda tasks name"
    parser = ArgumentParser(description=description, add_help=False)
    parser.add_argument("--name", default=name, type=str, help=nameHelp)
    parser.add_argument("--version", default=1, type=positiveInt, help="lambda version")
    if lambdaType == "handlers":
        parser.add_argument("--lambda-type", default="handlers", choices=["handlers", "standalone"], help="lambda type")
    parser.add_argument("--config", type=str, help="path to config")
    parser.add_argument(
        "--config-reload",
        "-cf",
        default=0,
        type=int,
        help="reload configuration in real time",
        const=True,
        nargs="?",
        choices=[0, 1],
    )
    parser.add_argument(
        "--luna-config",
        default=None,
        type=str,
        metavar="=",
        help="origin and api version of luna-configurator service for pulling setting from it "
        '(e.g. "http://127.0.0.1:5070/1")',
    )

    cmdArgs, _ = parser.parse_known_args()
    return cmdArgs
